module Manager.ModalManager exposing (..)
-- Url
import Url
-- Own
import Util.Util         exposing (MousePosition,Picture)
import Util.ShapeManager exposing (..)

type Modal
  = NoModal
  | ShowTable
  | ShowInput Shape String String
  | ShowShape Int Int
  | ShowJson
  | EditShape Int ShapeStyle
  | ShowManagePicture String (Maybe Url.Url) Int Int

type ModalMsg
  = CloseModal
  -- show Modal
  | OpenShowTable
  | OpenShowInput
  | OpenShowShape Int Int
  | OpenShowJson
  | OpenEditShape Int ShapeStyle String String
  | OpenManagePicture String String
  -- for input / delete Shapes
  | ResetShowInput
  | Check String
  | Input Char String
  | InputPoint

-- Update
updateModal : ModalMsg -> Modal -> Modal
updateModal msg modal =
  case msg of
    CloseModal  -> NoModal
    OpenShowTable    -> if (eq_String_Modal "ShowTable" modal)
                          then NoModal
                          else ShowTable
    OpenShowInput    -> if (eq_String_Modal "ShowInput" modal)
                          then NoModal
                          else ShowInput Empty "" ""
    OpenShowShape id edit -> if (id <= 0)
                               then NoModal
                               else ShowShape id edit
    OpenShowJson     -> ShowJson
    OpenEditShape id shapeStyle inputType value->
                        if (id <= 0)
                          then NoModal
                          else EditShape id (case inputType of
                                               "TA"  -> let values = String.split "\n" value
                                                        in if List.length values==10
                                                             then { shapeStyle
                                                                    | hoveredFillColor       = Maybe.withDefault "white" (List.head values)
                                                                    , hoveredFillOpacity     = Maybe.withDefault "0.5"   (List.head (List.drop 1 values))
                                                                    , hoveredStrokeColor     = Maybe.withDefault "red"   (List.head (List.drop 2 values))
                                                                    , hoveredStrokeWidth     = Maybe.withDefault "3"     (List.head (List.drop 3 values))
                                                                    , hoveredStrokeOpacity   = Maybe.withDefault "1.0"   (List.head (List.drop 4 values))
                                                                    , unHoveredFillColor     = Maybe.withDefault "white" (List.head (List.drop 5 values))
                                                                    , unHoveredFillOpacity   = Maybe.withDefault "0.0"   (List.head (List.drop 6 values))
                                                                    , unHoveredStrokeColor   = Maybe.withDefault "red"   (List.head (List.drop 7 values))
                                                                    , unHoveredStrokeWidth   = Maybe.withDefault "3"     (List.head (List.drop 8 values))
                                                                    , unHoveredStrokeOpacity = Maybe.withDefault "1.0"   (List.head (List.drop 9 values))
                                                                  }
                                                             else shapeStyle
                                               _     -> shapeStyle
                                            )
    OpenManagePicture iT value -> case modal of
                                    NoModal              -> ShowManagePicture value Nothing 0 0
                                    ShowTable            -> modal -- ERROR
                                    ShowShape _ _        -> modal -- ERROR
                                    ShowInput _ _ _      -> modal -- ERROR
                                    EditShape _ _        -> modal -- ERROR
                                    ShowJson             -> modal -- ERROR
                                    ShowManagePicture s url w h ->
                                      case iT of
                                        "i1" -> case Url.fromString value of
                                                 Just newUrl -> ShowManagePicture value (Just newUrl) w h
                                                 Nothing     -> ShowManagePicture value Nothing       w h
                                        "i2" -> ShowManagePicture s url (Maybe.withDefault 0 (String.toInt value)) h
                                        "i3" -> ShowManagePicture s url w (Maybe.withDefault 0 (String.toInt value))
                                        _    -> modal

    ResetShowInput -> ShowInput
                        (deleteCoordsFromShape
                          (getInputShapeFromModal modal)
                        ) "" ""
    Check shape -> if (eq_String_Modal "ShowInput" modal)
                     then if (eq_String_Shape shape (getInputShapeFromModal modal))
                            then ShowInput Empty                           "" ""
                            else ShowInput (shapeDefaultConstruktor shape) "" ""
                     else modal
    Input inputType value ->
      case modal of
        NoModal                  -> modal -- ERROR
        ShowTable                -> modal -- ERROR
        ShowShape _ _            -> modal -- ERROR
        ShowJson                 -> modal -- ERROR
        ShowInput iShape ix iy   -> updateInput inputType value modal "ShowInput" iShape ix iy {x = 0, y = 0}
        EditShape _ _            -> modal -- ERROR
        ShowManagePicture _ _ _ _-> modal -- ERROR
    InputPoint -> case modal of
                    NoModal                   -> modal -- ERROR
                    ShowTable                 -> modal -- ERROR
                    ShowShape _ _             -> modal -- ERROR
                    ShowJson                  -> modal -- ERROR
                    ShowInput iShape ix iy    -> ShowInput (Polygon ((ix,iy)::(getPointListFromPolygon iShape))) "" ""
                    EditShape _ _             -> modal -- ERROR
                    ShowManagePicture _ _ _ _ -> modal -- ERROR
updateInput : Char -> String -> Modal -> String -> Shape -> String -> String -> MousePosition -> Modal
updateInput inputType value modal modalType iShape ix iy mouse =
    case inputType of
      '1' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     _ b c   -> modalInputConstruktor modalType (Circle    value b c  ) ix iy mouse
        Rectangle  _ b c d -> modalInputConstruktor modalType (Rectangle value b c d) ix iy mouse
        Ellipse    _ b c d -> modalInputConstruktor modalType (Ellipse   value b c d) ix iy mouse
        Polygon    _       -> modalInputConstruktor modalType iShape               value iy mouse
      '2' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     a _ c   -> modalInputConstruktor modalType (Circle    a value c  ) ix iy mouse
        Rectangle  a _ c d -> modalInputConstruktor modalType (Rectangle a value c d) ix iy mouse
        Ellipse    a _ c d -> modalInputConstruktor modalType (Ellipse   a value c d) ix iy mouse
        Polygon    _       -> modalInputConstruktor modalType iShape               ix value mouse
      '3' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     a b _   -> modalInputConstruktor modalType (Circle    a b value  ) ix iy mouse
        Rectangle  a b _ d -> modalInputConstruktor modalType (Rectangle a b value d) ix iy mouse
        Ellipse    a b _ d -> modalInputConstruktor modalType (Ellipse   a b value d) ix iy mouse
        Polygon    _       -> modal -- ERROR
      '4' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     _ _ _   -> modal -- ERROR
        Rectangle  a b c _ -> modalInputConstruktor modalType (Rectangle a b c value) ix iy mouse
        Ellipse    a b c _ -> modalInputConstruktor modalType (Ellipse   a b c value) ix iy mouse
        Polygon    _       -> modal -- ERROR
      _   -> modal -- ERROR
-- Modal Construktors
modalInputConstruktor : String -> Shape -> String -> String -> MousePosition -> Modal
modalInputConstruktor modalType shape x y mouse =
    case modalType of
      "ShowInput" -> ShowInput shape x y
      _           -> NoModal
-- Operations on Modal
eq_String_Modal : String -> Modal -> Bool
eq_String_Modal string modal = string==(modalNameToString modal)
-- Get Methods : Parameter of Modal
getUrlFromModal : Modal -> Maybe Url.Url
getUrlFromModal modal =
    case modal of
      NoModal                     -> Nothing
      ShowTable                   -> Nothing
      ShowInput _ _ _             -> Nothing
      ShowShape _ _               -> Nothing
      ShowJson                    -> Nothing
      EditShape _ _               -> Nothing
      ShowManagePicture _ url _ _ -> url
getInputShapeFromModal : Modal -> Shape
getInputShapeFromModal modal =
    case modal of
      NoModal                   -> Empty
      ShowTable                 -> Empty
      ShowInput shape _ _       -> shape
      ShowShape _ _             -> Empty
      ShowJson                  -> Empty
      EditShape _ _             -> Empty
      ShowManagePicture _ _ _ _ -> Empty
-- Transform Modal to other Types
modalNameToString : Modal -> String
modalNameToString modal =
    case modal of
      NoModal                   -> "NoModal"
      ShowTable                 -> "ShowTable"
      ShowInput _ _ _           -> "ShowInput"
      ShowShape _ _             -> "ShowShape"
      ShowJson                  -> "ShowJson"
      EditShape _ _             -> "EditShape"
      ShowManagePicture _ _ _ _ -> "ShowManagePicture"